Add a new interface, walk_iomem_res_desc(), which walks through
the iomem table by identifying a target with @flags and @desc.
This interface provides the same functionality as walk_iomem_res(),
but does not use strcmp() to @name for better efficiency.

walk_iomem_res() is deprecated, but is maintained for backward
compatibility since walk_iomem_res_desc() requires 'desc' of
a target resource entry be initialized with its descriptor ID.

Cc: Andrew Morton <a...@linux-foundation.org>
Cc: Borislav Petkov <b...@alien8.de>
Cc: Dan Williams <dan.j.willi...@intel.com>
Signed-off-by: Toshi Kani <toshi.k...@hpe.com>
---
 include/linux/ioport.h |    3 ++
 kernel/resource.c      |   58 ++++++++++++++++++++++++++++++++++++++++++------
 2 files changed, 54 insertions(+), 7 deletions(-)

diff --git a/include/linux/ioport.h b/include/linux/ioport.h
index b7350c0..c4cd43a 100644
--- a/include/linux/ioport.h
+++ b/include/linux/ioport.h
@@ -269,6 +269,9 @@ extern int
 walk_system_ram_res(u64 start, u64 end, void *arg,
                    int (*func)(u64, u64, void *));
 extern int
+walk_iomem_res_desc(unsigned long desc, unsigned long flags, u64 start, u64 
end,
+                   void *arg, int (*func)(u64, u64, void *));
+extern int
 walk_iomem_res(char *name, unsigned long flags, u64 start, u64 end, void *arg,
               int (*func)(u64, u64, void *));
 
diff --git a/kernel/resource.c b/kernel/resource.c
index 52e6380..579b0e1 100644
--- a/kernel/resource.c
+++ b/kernel/resource.c
@@ -334,13 +334,14 @@ EXPORT_SYMBOL(release_resource);
 
 /*
  * Finds the lowest iomem reosurce exists with-in [res->start.res->end)
  * the caller must specify res->start, res->end, res->flags and "name".
  * If found, returns 0, res is overwritten, if not found, returns -1.
  * This walks through whole tree and not just first level children
  * until and unless first_level_children_only is true.
  */
-static int find_next_iomem_res(struct resource *res, char *name,
-                              bool first_level_children_only)
+static int find_next_iomem_res(struct resource *res, unsigned long desc,
+                               char *name, bool first_level_children_only)
 {
        resource_size_t start, end;
        struct resource *p;
@@ -360,6 +361,8 @@ static int find_next_iomem_res(struct resource *res, char 
*name,
        for (p = iomem_resource.child; p; p = next_resource(p, sibling_only)) {
                if ((p->flags & res->flags) != res->flags)
                        continue;
+               if ((desc != IORES_DESC_NONE) && (desc != p->desc))
+                       continue;
                if (name && strcmp(p->name, name))
                        continue;
                if (p->start > end) {
@@ -387,10 +390,51 @@ static int find_next_iomem_res(struct resource *res, char 
*name,
  * All the memory ranges which overlap start,end and also match flags and
  * name are valid candidates.
  *
+ * @desc: I/O resource descriptor. Use IORES_DESC_NONE to skip this check.
+ * @flags: I/O resource flags
+ * @start: start addr
+ * @end: end addr
+ *
+ * NOTE: For a new descriptor search, define a new IORES_DESC in
+ * <linux/ioport.h> and set it to 'desc' of a target resource entry.
+ */
+int walk_iomem_res_desc(unsigned long desc, unsigned long flags, u64 start,
+               u64 end, void *arg, int (*func)(u64, u64, void *))
+{
+       struct resource res;
+       u64 orig_end;
+       int ret = -1;
+
+       res.start = start;
+       res.end = end;
+       res.flags = flags;
+       orig_end = res.end;
+
+       while ((res.start < res.end) &&
+               (!find_next_iomem_res(&res, desc, NULL, false))) {
+               ret = (*func)(res.start, res.end, arg);
+               if (ret)
+                       break;
+               res.start = res.end + 1;
+               res.end = orig_end;
+       }
+
+       return ret;
+}
+
+/*
+ * Walks through iomem resources and calls func() with matching resource
+ * ranges. This walks through whole tree and not just first level children.
+ * All the memory ranges which overlap start,end and also match flags and
+ * name are valid candidates.
+ *
  * @name: name of resource
  * @flags: resource flags
  * @start: start addr
  * @end: end addr
+ *
+ * NOTE: This function is deprecated and should not be used in new code.
+ *       Use walk_iomem_res_desc(), instead.
  */
 int walk_iomem_res(char *name, unsigned long flags, u64 start, u64 end,
                void *arg, int (*func)(u64, u64, void *))
@@ -404,7 +448,7 @@ int walk_iomem_res(char *name, unsigned long flags, u64 
start, u64 end,
        res.flags = flags;
        orig_end = res.end;
        while ((res.start < res.end) &&
-               (!find_next_iomem_res(&res, name, false))) {
+               (!find_next_iomem_res(&res, IORES_DESC_NONE, name, false))) {
                ret = (*func)(res.start, res.end, arg);
                if (ret)
                        break;
@@ -433,7 +477,7 @@ int walk_system_ram_res(u64 start, u64 end, void *arg,
        res.flags = IORESOURCE_SYSTEM_RAM | IORESOURCE_BUSY;
        orig_end = res.end;
        while ((res.start < res.end) &&
-               (!find_next_iomem_res(&res, NULL, true))) {
+               (!find_next_iomem_res(&res, IORES_DESC_NONE, NULL, true))) {
                ret = (*func)(res.start, res.end, arg);
                if (ret)
                        break;
@@ -463,7 +507,7 @@ int walk_system_ram_range(unsigned long start_pfn, unsigned 
long nr_pages,
        res.flags = IORESOURCE_SYSTEM_RAM | IORESOURCE_BUSY;
        orig_end = res.end;
        while ((res.start < res.end) &&
-               (find_next_iomem_res(&res, NULL, true) >= 0)) {
+               (find_next_iomem_res(&res, IORES_DESC_NONE, NULL, true) >= 0)) {
                pfn = (res.start + PAGE_SIZE - 1) >> PAGE_SHIFT;
                end_pfn = (res.end + 1) >> PAGE_SHIFT;
                if (end_pfn > pfn)
@@ -334,13 +334,14 @@ EXPORT_SYMBOL(release_resource);
 /*
  * Finds the lowest iomem reosurce exists with-in [res->start.res->end)
- * the caller must specify res->start, res->end, res->flags and "name".
- * If found, returns 0, res is overwritten, if not found, returns -1.
+ * the caller must specify res->start, res->end, res->flags, and optionally
+ * desc and "name".  If found, returns 0, res is overwritten, if not found,
+ * returns -1.
  * This walks through whole tree and not just first level children
  * until and unless first_level_children_only is true.
--
To unsubscribe from this list: send the line "unsubscribe linux-kernel" in
the body of a message to majord...@vger.kernel.org
More majordomo info at  http://vger.kernel.org/majordomo-info.html
Please read the FAQ at  http://www.tux.org/lkml/

Reply via email to